@@ -43,16 +43,24 @@ significant_gene_similarity: float = 0.1
 #: :py:const:`dissolve_min_convincing_gene_fold_factor`.
 significant_gene_fold_factor: float = 3.0
 
-#: The generic minimal value (number of UMIs) we can say is "significant" given the technical noise.
-#: See
-#: py:const:`rare_min_gene_maximum`,
-#: py:const:`rare_min_cell_module_total`,
+#: Whether to use the absolute folds when considering fold factors. See
+#: :py:const:`deviants_abs_folds`,
+#: :py:const:`distinct_abs_folds`,
+#: :py:const:`inner_abs_folds`,
+#: :py:const:`outliers_abs_folds`,
+#: :py:const:`project_abs_folds`,
 #: and
-#: py:const:`cells_similarity_value_normalization`.
+#: :py:func:`inner_abs_folds`.
+abs_folds: bool = True
+
+#: The generic minimal value (number of UMIs) we can say is "significant" given the technical noise. See
+#: :py:const:`rare_min_gene_maximum`,
+#: :py:const:`rare_min_cell_module_total`,
+#: and
+#: :py:const:`cells_similarity_value_normalization`.
 significant_value: int = 7
 
-#: The generic minimal samples to use for downsampling the cells for some
-#: purpose. See
+#: The generic minimal samples to use for downsampling the cells for some purpose. See
 #: :py:const:`noisy_lonely_downsample_min_samples`,
 #: :py:const:`feature_downsample_min_samples`,
 #: and
@@ -103,12 +111,25 @@ logistics_location: float = 0.8
 #: :py:func:`metacells.utilities.computation.logistics`.
 logistics_slope: float = 0.5
 
-#: The generic target number of observations (cells) in a pile, allowing us to
+#: The minimal target number of observations (cells) in a pile, allowing us to
 #: directly compute groups (metacells) for it. See
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
 #: and
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
-target_pile_size: int = 10000
+min_target_pile_size: int = 10000
+
+#: The maximal target number of observations (cells) in a pile, allowing us to
+#: directly compute groups (metacells) for it. See
+#: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
+#: and
+#: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
+max_target_pile_size: int = 30000
+
+#: The target number of metacells computed in each pile. See
+#: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
+#: and
+#: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
+target_metacells_in_pile: int = 100
 
 #: The generic target total metacell size. See
 #: :py:const:`candidates_target_metacell_size`,
@@ -123,7 +144,7 @@ target_metacell_size: float = 160000
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`,
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`
 #: and
-#: py:func:`metacells.pipeline.collect.collect_metacells`.
+#: :py:func:`metacells.pipeline.collect.collect_metacells`.
 max_cell_size: Optional[float] = None
 
 #: The maximal cell size as a factor of the median cell size. See
@@ -131,7 +152,7 @@ max_cell_size: Optional[float] = None
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`,
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`
 #: and
-#: py:func:`metacells.pipeline.collect.collect_metacells`.
+#: :py:func:`metacells.pipeline.collect.collect_metacells`.
 max_cell_size_factor: Optional[float] = 2.0
 
 #: The genetic size of each cell for computing each metacell's size. See
@@ -392,13 +413,13 @@ rare_min_genes_of_modules: int = 4
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
 rare_min_cells_of_modules: int = min_metacell_cells
 
-#: The maximal mean number of cells in a random pile for a rare gene module to be considered rare.
-#: See
+#: The maximal mean number of cells (as a fraction of the mean metacell size) in a random pile for a rare gene module to
+#: be considered rare. See
 #: :py:const:`min_metacell_cells`,
 #: :py:func:`metacells.tools.rare.find_rare_gene_modules`
 #: and
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
-rare_max_cells_of_random_pile: int = min_metacell_cells * 4
+rare_max_cells_factor_of_random_pile: float = 0.5
 
 #: The minimal total UMIs of all the cells in a rare gene module (as a fraction
 #: of the :py:const:`target_metacell_size`). See
@@ -696,6 +717,10 @@ final_max_outliers_levels: Optional[int] = 1
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
 deviants_min_gene_fold_factor: float = significant_gene_fold_factor
 
+#: Whether to consider the absolute fold factor when computing deviant cells. See
+#: :py:func:`metacells.tools.distinct.find_deviant_cells`.
+deviants_abs_folds: bool = False
+
 #: The maximal fraction of genes to use to indicate cell are "deviants". See
 #: :py:func:`metacells.tools.deviants.find_deviant_cells`,
 #: :py:func:`metacells.pipeline.direct.compute_direct_metacells`,
@@ -775,10 +800,18 @@ rare_dissolve_min_convincing_size_factor: Optional[float] = None
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
 dissolve_min_convincing_gene_fold_factor: float = significant_gene_fold_factor
 
+#: Whether to consider the absolute fold factor when dissolving metacells. See
+#: :py:func:`metacells.tools.distinct.dissolve_metacells`.
+dissolve_abs_folds: bool = False
+
 #: The number of most-distinct genes to collect for each cell. See
 #: :py:func:`metacells.tools.distinct.find_distinct_genes`.
 distinct_genes_count: int = 20
 
+#: Whether to consider the absolute fold factor when collecting most-distinct genes for each cell. See
+#: :py:func:`metacells.tools.distinct.find_distinct_genes`.
+distinct_abs_folds: bool = abs_folds
+
 #: The normalization factor to use if/when computing the fractions of the data for UMAP.
 #: See
 #: :py:const:`metacells.parameters.target_metacell_size`
@@ -853,3 +886,150 @@ quality_min_gene_total: int = 40
 #: negative, is the fraction of the machine's total RAM to use as a safety buffer. See
 #: :py:func:`metacells.pipeline.divide_and_conquer.guess_max_parallel_piles`.
 max_gbs: float = -0.1
+
+#: Whether to compute projections based on the log of the data instead of the data itself. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_log_data: bool = True
+
+#: The normalization factor to use when computing fold factors for projecting a query onto an atlas. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_fold_normalization: float = 1e-5
+
+#: The minimal number of UMIs for a gene to be a potential cause to mark a metacell as dissimilar. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_min_significant_gene_value: float = 40
+
+#: The number of atlas candidates to consider when projecting a query onto an atlas. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_candidates_count: int = 50
+
+#: The minimal weight of an atlas metacell used for the projection of a query metacell. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_min_usage_weight: float = 1e-5
+
+#: The maximal fold factor of genes between the projection and the query metacell. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_max_projection_fold_factor: float = significant_gene_fold_factor
+
+#: Whether to consider the absolute fold factor when evaluating the projection of the query metacells . See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_abs_folds: bool = abs_folds
+
+#: The maximal fold factor of genes between the atlas metacells used for the projection of a query metacell. See
+#: :py:func:`metacells.tools.project.project_query_onto_atlas`.
+project_max_consistency_fold_factor: float = significant_gene_fold_factor - 1.0
+
+#: The minimal fold factor for a gene to be significant for metacell quality. See
+#: :py:func:`metacell.tools.compute_inner_fold_factors`.
+min_gene_inner_fold_factor: float = significant_gene_fold_factor
+
+#: The minimal fold factor for a gene entry in a metacell to be significant for metacell quality. See
+#: :py:func:`metacell.tools.compute_inner_fold_factors`.
+min_entry_inner_fold_factor: float = significant_gene_fold_factor - 1.0
+
+#: Whether to consider the absolute fold factor when evaluating the inner folds. See
+#: :py:func:`metacells.tools.distinct.compute_inner_fold_factors`.
+inner_abs_folds: bool = abs_folds
+
+#: The minimal fold factor for a gene to be significant for outliers. See
+#: :py:func:`metacell.tools.compute_outliers_fold_factors`.
+min_gene_outliers_fold_factor: float = significant_gene_fold_factor
+
+#: The minimal fold factor for a gene entry in a metacell to be significant for outliers. See
+#: :py:func:`metacell.tools.compute_outliers_fold_factors`.
+min_entry_outliers_fold_factor: float = significant_gene_fold_factor - 1.0
+
+#: Whether to consider the absolute fold factor when evaluating the outliers folds. See
+#: :py:func:`metacells.tools.distinct.compute_outliers_fold_factors`.
+outliers_abs_folds: bool = abs_folds
+
+#: The minimal fold factor for a gene entry in a metacell to be significant for metacell projection quality. See
+#: :py:func:`metacell.tools.compute_project_fold_factors`.
+min_entry_project_fold_factor: float = significant_gene_fold_factor - 1.0
+
+#: The normalization factor to use when computing log of fractions for finding the most similar group for outliers. See
+#: :py:func:`metacells.tools.quality.compute_outliers_matches`.
+outliers_fold_normalization: float = 1e-5
+
+#: Whether to ignore the forbidden genes of the atlas when computing projections. See
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+ignore_atlas_forbidden_genes: bool = True
+
+#: Whether to ignore the insignificant genes of the atlas when computing projections. See
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+ignore_atlas_insignificant_genes: bool = True
+
+#: Whether to ignore the insignificant genes of the query when computing projections. See
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+ignore_query_insignificant_genes: bool = False
+
+#: Whether to ignore the forbidden genes of the query when computing projections. See
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+ignore_query_forbidden_genes: bool = False
+
+#: The quantile of the gene value to use for the query gene expressions when looking for systematic genes. See
+#: :py:func:`metacells.tools.project.find_systematic_genes`,
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+systematic_low_gene_quantile: float = 0.05
+
+#: The quantile of the gene value to use for the atlas gene expressions when looking for systematic genes. See
+#: :py:func:`metacells.tools.project.find_systematic_genes`
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+systematic_high_gene_quantile: float = 0.95
+
+#: The minimal fraction of metacells where a gene has a high projection fold factor to mark the gene as biased.
+#: See :py:func:`metacells.tools.project.find_biased_genes`,
+#: :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+biased_min_metacells_fraction: float = 0.5
+
+#: The minimal fold between the maximal and minimal gene expression in metacells to be significant.
+#: See :py:func:`metacells.tools.high.find_significant_metacells_genes`.
+min_significant_metacells_gene_range_fold_factor: float = 2.0
+
+#: The normalization factor to use after computing the fractions of the data for
+#: computing metacell gene range folds. See
+#: :py:func:`metacells.tools.high.find_significant_metacells_genes`.
+metacells_gene_range_normalization: float = 1e-5
+
+#: The minimal maximal gene expression in metacells to be significant.
+#: See :py:func:`metacells.tools.high.find_significant_metacells_genes`.
+min_significant_metacells_gene_fraction: float = 1e-4
+
+#: Whether to renormalize the query to account for missing atlas genes when computing projections.
+#: See :py:func:`metacells.pipeline.projection.direct_projection_pipeline`
+#: and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+project_renormalize_query: bool = False
+
+#: The minimal correlation between observed and projected genes for considering linear correction of the query gene
+#: value.
+#: See :py:func:`metacells.pipeline.projection.projection_pipeline`.
+project_min_corrected_gene_correlation: float = 0.8
+
+#: The minimal strength of the correction between the mean query and projected mean value (for correlated genes).
+#: See :py:func:`metacells.pipeline.projection.projection_pipeline`.
+project_min_corrected_gene_factor: float = 0.15
+
+#: The m aximal correlation between observed and projected genes for ignoring the gene as uncorrelated.
+#: See :py:func:`metacells.pipeline.projection.projection_pipeline`.
+project_max_uncorrelated_gene_correlation: float = 0.5
+
+#: The maximal number of deviant genes allowed for saying a query is similar to the projection in the atlas.
+#: See :py:func:`metacells.tools.quality.compute_similar_query_metacells`
+# and :py:func:`metacells.pipeline.projection.projection_pipeline`.
+project_max_dissimilar_genes: int = 3
+
+#: Whether to add a pseudo-gene to the query to renormalize its total UMIs so that the fractions of the common genes
+#: would be as expected. See :py:func:`metacells.tools.project.renormalize_query_by_atlas` and
+#: :py:func:`metacells.pipeline.projection.projection_pipeline`.
+renormalize_query_by_atlas: bool = True
+
+#: The quantile of each gene's normalized variance across the metacells to use for the overall gene's variability.
+#: See :py:func:`metacells.tools.quality.compute_type_gene_normalized_variance`.
+type_gene_normalized_variance_quantile: float = 0.95
