@@ -3,6 +3,7 @@ Candidates
 ----------
 """
 
+import sys
 from math import ceil
 from math import floor
 from typing import Optional
@@ -13,10 +14,12 @@ from typing import Union
 import numpy as np
 from anndata import AnnData  # type: ignore
 
-import metacells.extensions as xt  # type: ignore
 import metacells.parameters as pr
 import metacells.utilities as ut
 
+if "sphinx" not in sys.argv[0]:
+    import metacells.extensions as xt  # type: ignore
+
 __all__ = [
     "compute_candidate_metacells",
     "choose_seeds",
@@ -120,9 +123,9 @@ def compute_candidate_metacells(  # pylint: disable=too-many-statements,too-many
 
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
 
-    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
+    assert ut.is_layout(outgoing_edge_weights, "row_major")
     incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
-    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
+    assert ut.is_layout(incoming_edge_weights, "column_major")
 
     assert outgoing_edge_weights.data.dtype == "float32"
     assert outgoing_edge_weights.indices.dtype == "int32"
@@ -332,7 +335,7 @@ def _optimize_split_communities(
             return cold_temperature, score
 
 
-def _split_communities(  # pylint: disable=too-many-statements
+def _split_communities(
     *,
     outgoing_edge_weights: ut.CompressedMatrix,
     community_of_nodes: ut.NumpyVector,
@@ -370,36 +373,38 @@ def _split_communities(  # pylint: disable=too-many-statements
             cancelled_communities.add(community_index)
             continue
 
-        if max_split_min_cut_strength is None:
-            continue
+    if len(cancelled_communities) == 0 and max_split_min_cut_strength is not None:
+        for community_index in range(communities_count):
+            community_mask = community_of_nodes == community_index
 
-        community_indices = tuple(np.where(community_mask)[0])
-        if community_indices in atomic_candidates:
-            continue
+            community_indices = tuple(np.where(community_mask)[0])
+            if community_indices in atomic_candidates:
+                continue
+
+            did_cut, did_split = _min_cut_community(
+                outgoing_edge_weights=outgoing_edge_weights,
+                community_of_nodes=community_of_nodes,
+                cut_community_index=community_index,
+                max_split_min_cut_strength=max_split_min_cut_strength,
+                min_cut_seed_cells=min_cut_seed_cells,
+                must_complete_cover=must_complete_cover,
+                new_community_index=next_new_community_index,
+            )
+
+            if not did_cut:
+                atomic_candidates.add(community_indices)
+                continue
 
-        did_cut, did_split = _min_cut_community(
-            outgoing_edge_weights=outgoing_edge_weights,
-            community_of_nodes=community_of_nodes,
-            cut_community_index=community_index,
-            max_split_min_cut_strength=max_split_min_cut_strength,
-            min_cut_seed_cells=min_cut_seed_cells,
-            must_complete_cover=must_complete_cover,
-            new_community_index=next_new_community_index,
-        )
-        if did_cut:
             cut_communities_count += 1
             ut.logger().debug(
                 "community: %s nodes: %s size: %s is too divided",
                 community_index,
                 np.sum(community_mask),
-                community_size,
+                np.sum(node_sizes[community_mask]),
             )
             if did_split:
                 split_communities_count += 1
                 next_new_community_index += 1
-        else:
-            assert community_indices is not None
-            atomic_candidates.add(community_indices)
 
     cancelled_communities_count = len(cancelled_communities)
     modified_communities_count = cut_communities_count + cancelled_communities_count
@@ -577,10 +582,10 @@ def choose_seeds(
     size = edge_weights.shape[0]
 
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
-    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
+    assert ut.is_layout(outgoing_edge_weights, "row_major")
 
     incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
-    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
+    assert ut.is_layout(incoming_edge_weights, "column_major")
 
     if seed_of_cells is None:
         seed_of_cells = np.full(size, -1, dtype="int32")
@@ -672,10 +677,10 @@ def optimize_partitions(
     course no claim is made of achieving the global maximum of the goal function.
     """
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
-    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
+    assert ut.is_layout(outgoing_edge_weights, "row_major")
 
     incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
-    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
+    assert ut.is_layout(incoming_edge_weights, "column_major")
     return _optimize_partitions(
         outgoing_edge_weights=outgoing_edge_weights,
         incoming_edge_weights=incoming_edge_weights,
@@ -741,10 +746,10 @@ def score_partitions(
     """
     assert str(partition_of_nodes.dtype) == "int32"
     outgoing_edge_weights = ut.mustbe_compressed_matrix(edge_weights)
-    assert ut.matrix_layout(outgoing_edge_weights) == "row_major"
+    assert ut.is_layout(outgoing_edge_weights, "row_major")
 
     incoming_edge_weights = ut.mustbe_compressed_matrix(ut.to_layout(outgoing_edge_weights, layout="column_major"))
-    assert ut.matrix_layout(incoming_edge_weights) == "column_major"
+    assert ut.is_layout(incoming_edge_weights, "column_major")
 
     with ut.unfrozen(partition_of_nodes):
         with ut.timed_step(".score"):
