@@ -273,7 +273,7 @@ def _rank_outgoing(similarity: ut.NumpyMatrix) -> ut.NumpyMatrix:
 
     np.fill_diagonal(similarity, min_similarity - 1)
 
-    assert ut.matrix_layout(similarity) == "row_major"
+    assert ut.is_layout(similarity, "row_major")
     outgoing_ranks = ut.rank_matrix_by_layout(similarity, ascending=False)
     assert np.sum(np.diagonal(outgoing_ranks) == size) == size
     return outgoing_ranks
@@ -391,7 +391,7 @@ def _weigh_edges(pruned_ranks: ut.CompressedMatrix) -> ut.CompressedMatrix:
 
 def _assert_proper_compressed(matrix: ut.CompressedMatrix, layout: str) -> None:
     assert sp.issparse(matrix)
-    assert ut.matrix_dtype(matrix) == "float32"
+    assert ut.shaped_dtype(matrix) == "float32"
     assert matrix.getformat() == layout
     assert matrix.has_sorted_indices
     assert matrix.has_canonical_format
