@@ -324,7 +324,7 @@ def has_data(
         if layout is None:
             return True
         matrix: utt.Matrix = adata.X
-        return utt.matrix_layout(matrix) == layout or f"vo:__x__:{layout}" in derived
+        return utt.is_layout(matrix, layout) or f"vo:__x__:{layout}" in derived
 
     for per, annotations in (
         ("vo", adata.layers),
@@ -338,7 +338,7 @@ def has_data(
         if layout is None:
             return True
         matrix = annotations[name]
-        return utt.matrix_layout(matrix) == layout or f"{per}:{name}:{layout}" in derived
+        return utt.is_layout(matrix, layout) or f"{per}:{name}:{layout}" in derived
 
     assert layout is None
 
@@ -1109,6 +1109,13 @@ def set_vo_data(
     else:
         adata.layers[name] = data
 
+    if hasattr(adata, "__derived__"):
+        derived = getattr(adata, "__derived__")
+        for layout in ["column_major", "row_major"]:
+            layout_name = f"vo:{name}:{layout}"
+            if layout_name in derived:
+                del derived[layout_name]
+
 
 def _unknown_data(adata: AnnData, name: str, per: Optional[str] = None) -> KeyError:
     texts = ["unknown"]
