@@ -18,6 +18,7 @@ analysis pipeline.
 """
 
 import re
+import sys
 from math import ceil
 from math import floor
 from math import sqrt
@@ -41,11 +42,13 @@ import numpy as np
 import pandas as pd  # type: ignore
 import scipy.sparse as sp  # type: ignore
 
-import metacells.extensions as xt  # type: ignore
 import metacells.utilities.documentation as utd
 import metacells.utilities.timing as utm
 import metacells.utilities.typing as utt
 
+if "sphinx" not in sys.argv[0]:
+    import metacells.extensions as xt  # type: ignore
+
 __all__ = [
     "allow_inefficient_layout",
     "to_layout",
@@ -88,6 +91,7 @@ __all__ = [
     "rank_matrix_by_layout",
     "bincount_vector",
     "most_frequent",
+    "highest_weight",
     "fraction_of_grouped",
     "downsample_matrix",
     "downsample_vector",
@@ -105,6 +109,7 @@ __all__ = [
     "group_piles",
     "represent",
     "min_cut",
+    "sparsify_matrix",
 ]
 
 
@@ -303,13 +308,13 @@ def _ensure_per(matrix: utt.Matrix, per: Optional[str]) -> str:
 
 
 def _ensure_layout_for(operation: str, matrix: utt.Matrix, per: Optional[str], allow_inefficient: bool = True) -> None:
-    layout = utt.matrix_layout(matrix)
-    if layout == f"{per}_major":
+    if utt.is_layout(matrix, f"{per}_major"):
         return
 
     if not ALLOW_INEFFICIENT_LAYOUT:
         allow_inefficient = False
 
+    layout = utt.matrix_layout(matrix)
     if layout is not None:
         operating_on_matrix_of_wrong_layout = f"{operation} of {per}s of a matrix with {layout} layout"
         if not allow_inefficient:
@@ -459,15 +464,23 @@ def cross_corrcoef_rows(
     """
     first_matrix = utt.mustbe_numpy_matrix(first_matrix)
     second_matrix = utt.mustbe_numpy_matrix(second_matrix)
-    assert utt.matrix_layout(first_matrix) == "row_major"
-    assert utt.matrix_layout(second_matrix) == "row_major"
+    assert utt.is_layout(first_matrix, "row_major")
+    assert utt.is_layout(second_matrix, "row_major")
     assert first_matrix.shape[1] == second_matrix.shape[1]
     assert first_matrix.dtype == second_matrix.dtype
 
+    workaround = first_matrix.shape[0] == 1
+    if workaround:
+        first_matrix = np.concatenate([first_matrix, first_matrix])
+
     extension_name = f"cross_correlate_dense_{first_matrix.dtype}_t"
     result = np.empty((first_matrix.shape[0], second_matrix.shape[0]), dtype="float32")
     extension = getattr(xt, extension_name)
     extension(first_matrix, second_matrix, result)
+
+    if workaround:
+        result = result[0:1, :]
+
     return result
 
 
@@ -477,7 +490,7 @@ def pairs_corrcoef_rows(
     second_matrix: utt.NumpyMatrix,
     *,
     reproducible: bool,  # pylint: disable=unused-argument
-) -> utt.NumpyMatrix:
+) -> utt.NumpyVector:
     """
     Similar to for ``numpy.corrcoef``, but computes the correlations between each row of the
     ``first_matrix`` and each matching row of the ``second_matrix``. Both matrices must be dense, in
@@ -500,8 +513,8 @@ def pairs_corrcoef_rows(
     """
     first_matrix = utt.mustbe_numpy_matrix(first_matrix)
     second_matrix = utt.mustbe_numpy_matrix(second_matrix)
-    assert utt.matrix_layout(first_matrix) == "row_major"
-    assert utt.matrix_layout(second_matrix) == "row_major"
+    assert utt.is_layout(first_matrix, "row_major")
+    assert utt.is_layout(second_matrix, "row_major")
     assert first_matrix.shape == second_matrix.shape
     assert first_matrix.dtype == second_matrix.dtype
 
@@ -569,8 +582,8 @@ def cross_logistics_rows(
     """
     first_matrix = utt.mustbe_numpy_matrix(first_matrix)
     second_matrix = utt.mustbe_numpy_matrix(second_matrix)
-    assert utt.matrix_layout(first_matrix) == "row_major"
-    assert utt.matrix_layout(second_matrix) == "row_major"
+    assert utt.is_layout(first_matrix, "row_major")
+    assert utt.is_layout(second_matrix, "row_major")
     assert first_matrix.shape[1] == second_matrix.shape[1]
     assert first_matrix.dtype == second_matrix.dtype
 
@@ -596,8 +609,8 @@ def pairs_logistics_rows(
     """
     first_matrix = utt.mustbe_numpy_matrix(first_matrix)
     second_matrix = utt.mustbe_numpy_matrix(second_matrix)
-    assert utt.matrix_layout(first_matrix) == "row_major"
-    assert utt.matrix_layout(second_matrix) == "row_major"
+    assert utt.is_layout(first_matrix, "row_major")
+    assert utt.is_layout(second_matrix, "row_major")
     assert first_matrix.shape == second_matrix.shape
     assert first_matrix.dtype == second_matrix.dtype
 
@@ -1387,7 +1400,7 @@ def variance_per(matrix: utt.Matrix, *, per: Optional[str]) -> utt.NumpyVector:
 
 
 @utm.timed_call()
-def normalized_variance_per(matrix: utt.Matrix, *, per: Optional[str]) -> utt.NumpyVector:
+def normalized_variance_per(matrix: utt.Matrix, *, per: Optional[str], zero_value: float = 1.0) -> utt.NumpyVector:
     """
     Get the normalized variance (variance / mean) ``per`` (``row`` or ``column``) of some ``matrix``.
 
@@ -1395,6 +1408,8 @@ def normalized_variance_per(matrix: utt.Matrix, *, per: Optional[str]) -> utt.Nu
     efficient direction is used based on the matrix layout. Otherwise it must be one of ``row`` or
     ``column``, and the matrix must be in the appropriate layout (``row_major`` operating on rows,
     ``column_major`` for operating on columns).
+
+    If all the values are zero, writes the ``zero_value`` (default: {zero_value}) into the result.
     """
     variance_per_element = variance_per(matrix, per=per)
     mean_per_element = mean_per(matrix, per=per)
@@ -1402,7 +1417,7 @@ def normalized_variance_per(matrix: utt.Matrix, *, per: Optional[str]) -> utt.Nu
     result = np.reciprocal(mean_per_element, where=~zeros_mask)
     result[zeros_mask] = 0
     result *= variance_per_element
-    result[zeros_mask] = 1
+    result[zeros_mask] = zero_value
     return result
 
 
@@ -1610,6 +1625,17 @@ def most_frequent(vector: utt.Vector) -> Any:
     return unique[maxpos]
 
 
+@utm.timed_call()
+def highest_weight(weights: utt.Vector, vector: utt.Vector) -> Any:
+    """
+    Return the value with the highest total ``weight`` in a ``vector``.
+    """
+    unique, positions = np.unique(utt.to_numpy_vector(vector), return_inverse=True)
+    counts = np.bincount(positions, weights=weights)
+    maxpos = np.argmax(counts)
+    return unique[maxpos]
+
+
 @utm.timed_call()
 def fraction_of_grouped(value: Any) -> Callable[[utt.Vector], Any]:
     """
@@ -1929,13 +1955,13 @@ def sum_groups(
 
     sparse = utt.maybe_sparse_matrix(matrix)
     if sparse is not None:
-        if utt.matrix_layout(matrix) == efficient_layout:
+        if utt.is_layout(matrix, efficient_layout):
             timed_step = ".compressed-efficient"
         else:
             timed_step = ".compressed-inefficient"
     else:
         matrix = utt.to_numpy_matrix(matrix, only_extract=True)
-        if utt.matrix_layout(matrix) == efficient_layout:
+        if utt.is_layout(matrix, efficient_layout):
             timed_step = ".dense-efficient"
         else:
             timed_step = ".dense-inefficient"
@@ -1947,7 +1973,7 @@ def sum_groups(
 
     with utm.timed_step(timed_step):
         utm.timed_parameters(groups=groups_count, entities=matrix.shape[0], elements=matrix.shape[1])
-        results = np.empty((groups_count, matrix.shape[1]), dtype=utt.matrix_dtype(matrix))
+        results = np.empty((groups_count, matrix.shape[1]), dtype=utt.shaped_dtype(matrix))
 
         for group_index in range(groups_count):
             group_mask = groups == group_index
@@ -2154,7 +2180,13 @@ def represent(
     objective = cvxpy.norm(goal - variables @ basis, 2)
 
     problem = cvxpy.Problem(cvxpy.Minimize(objective), constraints)
-    result = problem.solve()
+    try:
+        result = problem.solve(solver="SCS")
+    except cvxpy.error.SolverError:
+        try:
+            result = problem.solve(solver="ECOS")
+        except cvxpy.error.SolverError:
+            result = problem.solve(solver="QSQP")
 
     if result is None:
         return None
@@ -2243,3 +2275,38 @@ def min_cut(  # pylint: disable=too-many-branches,too-many-statements
         cut_strength = None
 
     return (cut, cut_strength)
+
+
+@utm.timed_call()
+def sparsify_matrix(
+    full: utt.ProperMatrix,
+    min_column_max_value: float,
+    min_entry_value: float,
+    abs_values: bool,
+) -> utt.CompressedMatrix:
+    """
+    Given a full matrix, return a sparse matrix such that all non-zero entries are at least ``min_entry_value``, and
+    columns that have no value above ``min_column_max_value`` are set to all-zero. If ``abs_values`` consider the
+    absolute values when comparing to the thresholds.
+    """
+    assert 0 <= min_entry_value <= min_column_max_value
+
+    if abs_values:
+        comparable = np.abs(full)  # type: ignore
+    else:
+        comparable = full
+
+    max_per_column = max_per(comparable, per="column")
+    low_columns = max_per_column < min_column_max_value
+
+    sparse_comparable = utt.maybe_compressed_matrix(comparable)
+    if sparse_comparable is None:
+        low_entries = comparable < min_entry_value
+    else:
+        low_entries = ~utt.to_numpy_matrix(comparable >= min_entry_value)
+
+    lil = sp.lil_matrix(full)
+    lil[:, low_columns] = 0
+    lil[low_entries] = 0
+
+    return sp.csr_matrix(lil)
