@@ -190,7 +190,7 @@ class ShortLoggingFormatter(LoggingFormatter):
         return LoggingFormatter.format(self, record)
 
 
-# ' Global logger object.
+# Global logger object.
 LOG: Optional[Logger] = None
 
 
@@ -417,10 +417,10 @@ def _format_value(  # pylint: disable=too-many-return-statements,too-many-branch
         return None
 
     checksum = ""
-    #   if utt.is_1d(value) and 'U' not in str(value.dtype) and 'o' not in str(value.dtype):
-    #       checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
-    #   elif utt.is_2d(value):
-    #       checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
+    # if utt.is_1d(value) and 'U' not in str(value.dtype) and 'o' not in str(value.dtype):
+    #    checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
+    # elif utt.is_2d(value):
+    #    checksum = ' checksum: %.20e' % utt.shaped_checksum(value)
 
     if formatter is not None:
         value = formatter(value)
@@ -448,23 +448,29 @@ def _format_value(  # pylint: disable=too-many-return-statements,too-many-branch
             return f"{value:g} (reproducible)"
         if "rank" in name:
             return f"{value:g}"
-        if "fold" in name:
+        if "fold" in name and "nnz" not in name and (-10 <= value <= 10):
             return fold_description(value)
         if "fraction" in name or "quantile" in name:
             return fraction_description(value)
-        if "factor" in name:
+        if "factor" in name and "nnz" not in name:
             return f"X {value:.4g}"
         return f"{value:g}" + checksum
 
     if hasattr(value, "__qualname__"):
         return getattr(value, "__qualname__") + checksum
 
-    if isinstance(value, (pd.Series, np.ndarray)) and value.ndim == 1 and value.dtype == "bool":
+    if (
+        isinstance(value, (pd.DataFrame, pd.Series, np.ndarray))
+        and value.ndim in (1, 2)
+        and utt.shaped_dtype(value) == "bool"
+    ):
         return mask_description(value) + checksum
 
     if hasattr(value, "ndim"):
         if value.ndim == 2:
-            text = f"{value.__class__.__name__} {value.shape[0]} X {value.shape[1]} {utt.matrix_dtype(value)}s"
+            text = f"{value.__class__.__name__} {value.shape[0]} X {value.shape[1]} {utt.shaped_dtype(value)}s"
+            if hasattr(value, "nnz"):
+                text += f" ({value.nnz} > 0)"
             return text + checksum
 
         if value.ndim == 1:
@@ -483,6 +489,9 @@ def _format_value(  # pylint: disable=too-many-return-statements,too-many-branch
         ]
         return f'[ {", ".join(texts)} ]' + checksum
 
+    if isinstance(value, dict):
+        return str(value) + checksum
+
     raise RuntimeError(f"unknown parameter type: {value.__class__} value: {value}")  #
 
 
@@ -718,17 +727,24 @@ def groups_description(groups: Union[utt.Vector, str]) -> str:
     )
 
 
-def mask_description(mask: Union[str, utt.Vector]) -> str:
+def mask_description(mask: Union[str, utt.Vector, utt.Matrix]) -> str:
     """
     Return a string for logging a boolean mask.
     """
     if isinstance(mask, str):
         return mask
 
-    mask = utt.to_numpy_vector(mask)
     if mask.dtype == "bool":
-        return ratio_description(mask.size, "bool", np.sum(mask > 0), "true")
-    return ratio_description(mask.size, str(mask.dtype), np.sum(mask > 0), "positive")
+        value = "true"
+    else:
+        value = "positive"
+
+    if mask.ndim == 2:
+        return f"{mask.shape[0]} X {mask.shape[1]} " + ratio_description(
+            mask.size, str(mask.dtype), np.sum(mask > 0), value
+        )
+
+    return ratio_description(mask.size, str(mask.dtype), np.sum(mask > 0), value)
 
 
 def ratio_description(denominator: float, element: str, numerator: float, condition: str) -> str:
