@@ -125,7 +125,7 @@ __all__ = [
     "SPARSE_SLOW_FORMAT",
     "LAYOUT_OF_AXIS",
     "PER_OF_AXIS",
-    "matrix_dtype",
+    "shaped_dtype",
     "matrix_layout",
     "is_layout",
     "is_contiguous",
@@ -249,7 +249,7 @@ class CompressedMatrix(SparseMatrix, Protocol):
     """
     A ``mypy`` type for sparse CSR/CSC 2-dimensional data.
 
-    Should have been ``CompressedMatrix = sp.compressed._cs_matrix``.
+    Should have been ``CompressedMatrix = sp..._cs_matrix``.
     """
 
     indices: np.ndarray
@@ -411,7 +411,7 @@ def maybe_compressed_matrix(shaped: Any) -> Optional[CompressedMatrix]:
     """
     Return ``shaped`` as a :py:const:`CompressedMatrix`, if it is one.
     """
-    if isinstance(shaped, sp.compressed._cs_matrix):  # pylint: disable=protected-access
+    if isinstance(shaped, (sp.csr_matrix, sp.csc_matrix)):
         return shaped
     return None
 
@@ -473,7 +473,7 @@ def mustbe_compressed_matrix(shaped: Any) -> CompressedMatrix:
     """
     Return ``shaped`` as a :py:const:`CompressedMatrix`, asserting it must be one.
     """
-    assert isinstance(shaped, sp.compressed._cs_matrix)  # pylint: disable=protected-access
+    assert isinstance(shaped, (sp.csr_matrix, sp.csc_matrix))
     return shaped
 
 
@@ -802,6 +802,9 @@ def is_layout(matrix: Matrix, layout: Optional[str]) -> bool:
     if layout is None:
         return True
 
+    if matrix.shape[0] == 1 or matrix.shape[1] == 1:
+        return True
+
     sparse = maybe_sparse_matrix(matrix)
     if sparse is not None:
         return sparse.getformat() == SPARSE_FAST_FORMAT[layout]
@@ -810,24 +813,29 @@ def is_layout(matrix: Matrix, layout: Optional[str]) -> bool:
     return dense.flags[DENSE_FAST_FLAG[layout]]
 
 
-def matrix_dtype(proper: ProperMatrix) -> str:
+def shaped_dtype(shaped: Shaped) -> str:
     """
-    Return the data type of the element of a proper matrix.
+    Return the data type of the element of shaped data.
     """
-    frame = maybe_pandas_frame(proper)
+    frame = maybe_pandas_frame(shaped)
     if frame is not None:
-        proper = frame.values
-        if isinstance(proper, pd.core.arrays.categorical.Categorical):
-            proper = np.array(proper)
+        shaped = frame.values
 
-    dense = maybe_numpy_matrix(proper)
-    if dense is not None:
-        return str(dense.dtype)
+    compressed = maybe_compressed_matrix(shaped)
+    if compressed is not None:
+        shaped = compressed.data
 
-    compressed = maybe_compressed_matrix(proper)
-    assert compressed is not None
+    series = maybe_pandas_series(shaped)
+    if series is not None:
+        shaped = series.values
 
-    return str(compressed.data.dtype)
+    if isinstance(shaped, pd.core.arrays.categorical.Categorical):
+        shaped = np.array(shaped)
+
+    if isinstance(shaped, np.ndarray):
+        return str(shaped.dtype)
+
+    raise AssertionError(f"unexpected shaped type: {shaped.__class__.__qualname__}")
 
 
 def matrix_layout(matrix: Matrix) -> Optional[str]:
